@@ -1,3 +1,4 @@
+import os
 from dataclasses import dataclass
 from typing import Any, Dict, Optional, Type
 
@@ -22,20 +23,27 @@ class BaseSnatch:
     For each Datasource, you`ll need to subclass this class
     overriding the following properties:
 
-    * serializer_class (Schema):    The Serializer Class used to deserialize
+    * serializer_class (Schema)     The Serializer Class used to deserialize
                                     backend response dict to DataSource Data Object.
                                     Must be a Marshmallow schema.
-    * default_timeout (int):        Default timeout in seconds, for wait backend
+    * default_timeout (int)         Default timeout in seconds, for wait backend
                                     response for pending prospections.
                                     Please make sure you give enough
                                     time to backend complete prospect the Datasource.
-    * default_max_days (int):       Default max days for expiration check. All
+    * default_max_days (int)        Default max days for expiration check. All
                                     prospections with prospect_date older than
                                     max_days will be marked as EXPIRED.
-    * authorization_token (str):    The Datasource Authorization Token.
-    * tax_id (str):                 The TaxId prospected. Must be a valid CNPJ,
+    * authorization_token (str)     The Datasource Authorization Token.
+    * tax_id (str)                  The TaxId prospected. Must be a valid CNPJ,
                                     formatted or not.
-    * base_url (str):               DataSource base url.
+    * base_url (str)                DataSource base url.
+    * default_max_periods (int)     Max number of periods for expiration check.
+                                    All prospections with prospect month older than
+                                    default_max_periods will be marked as EXPIRED.
+    * use_env                       Use specific environment. Default: `production`.
+    * use_log                       Use specific Log Level for this DataSource.
+                                    Default: `INFO`. Can override with
+                                    `LOGURU_LEVEL` env variable.
     """
 
     serializer_class: Optional[Type[Schema]] = None
@@ -45,13 +53,14 @@ class BaseSnatch:
     tax_id: Optional[str] = None
     base_url_key: Optional[str] = None
     settings: Optional[Cut] = None
+    default_max_periods: Optional[int] = None
+    use_env: Optional[str] = None
+    use_log: str = "INFO"
 
     def __post_init__(self):
-        self.settings = get_settings()
-        cnpj = CNPJ()
-        if not cnpj.validate(self.tax_id):
-            raise ValueError("Invalid TaxId")
-        self.tax_id = only_numbers(self.tax_id)
+        self.settings = get_settings(
+            environment=self.use_env, log_level=self.logger_level
+        )
 
     @property
     def authorization_token(self) -> str:
@@ -68,9 +77,14 @@ class BaseSnatch:
             "Authorization": f"Token {self.authorization_token}",
         }
 
+    @property
+    def logger_level(self) -> str:
+        return os.getenv("LOGURU_LEVEL", self.use_log)
+
     def _waiter(self, timeout: Optional[int] = None):
         if not timeout:
             timeout = self.default_timeout
+        logger.info(f"Waiting data for {self.tax_id}. Timeout is: {timeout} seconds...")
         max_waiting_time = arrow.utcnow().shift(seconds=timeout)
         now = arrow.utcnow()
         next_check = now
@@ -84,7 +98,9 @@ class BaseSnatch:
                     "CONN_ERROR",
                     "NOT_FOUND",
                 ]:
-                    logger.info(f"Data Received - Status is: {data.integration_status}")
+                    logger.debug(
+                        f"Data Received - Status is: {data.integration_status}"
+                    )
                     return data
             now = arrow.utcnow()
         return self._format_timeout_response("/last_prospect")
@@ -97,10 +113,12 @@ class BaseSnatch:
 
     def _last_prospect(self, timeout: int) -> BaseData:
         url = f"/api/companies/{self.tax_id}/last_prospect/"
+        logger.debug(f"Calling {url}...")
         return self._request_and_create_data(url=url, timeout=timeout)
 
     def _last_valid(self, timeout: int) -> BaseData:
         url = f"/api/companies/{self.tax_id}/last_valid/"
+        logger.info(f"Calling {url}...")
         return self._request_and_create_data(url=url, timeout=timeout)
 
     def _format_response(self, data: Dict[Any, Any]) -> BaseData:
@@ -147,7 +165,7 @@ class BaseSnatch:
         try:
             method = "POST" if payload else "GET"
             full_url = f"{self.base_url}{url}"
-            logger.info(f"Start {method} request to {full_url} ...")
+            logger.debug(f"Start {method} request to {full_url} ...")
             response = requests.request(
                 method=method,
                 url=full_url,
@@ -164,23 +182,35 @@ class BaseSnatch:
             return self._format_timeout_response(error.request.url)
 
     @time_it
-    def last_valid(self, timeout: Optional[int] = None):
+    def last_valid(self, tax_id: str, timeout: Optional[int] = None):
         """Return last valid DataSource integration for TaxId.
 
         It will check for the last valid Datasource Integration
         for selected TaxId. The last valid integration is the
         last one with "SUCCESS" status, regardless prospect time.
 
+        :param tax_id: Valid TaxId to be prospected, with or without punctuation.
         :param timeout: Time in seconds before timeout connection. Default: 60 seconds.
         :return: DataSource Data Object
         """
+        self.tax_id = tax_id
+        cnpj = CNPJ()
+        if not cnpj.validate(self.tax_id):
+            raise ValueError("Invalid TaxId")
+        self.tax_id = only_numbers(self.tax_id)
+
         if not timeout:
             timeout = self.default_timeout
+
         return self._last_valid(timeout=timeout)
 
     @time_it
     def get_data(
-        self, timeout: Optional[int] = None, max_days_old: Optional[int] = None
+        self,
+        tax_id: str,
+        timeout: Optional[int] = None,
+        max_days_old: Optional[int] = None,
+        max_periods_old: Optional[int] = None,
     ):
         """Get or Request DataSource Integration Data.
 
@@ -189,7 +219,8 @@ class BaseSnatch:
         1. Check if exists a valid integration for
         selected TaxId in backend. A valid integration
         is a successfully integration (status: SUCCESS) with
-        prospect date less or equal the `max_days_old` informed.
+        prospect date less or equal the `max_days_old` informed,
+        or current period is `max_days_periods` greater than prospect period.
 
         2. If integration does not exist,
         has EXPIRED or has an status "ERROR",
@@ -198,25 +229,41 @@ class BaseSnatch:
         3. If integration was started or already PENDING/RUNNING,
         wait `timeout` seconds for the backend response.
 
+        :param tax_id: Valid TaxId to be prospected, with or without punctuation.
         :param max_days_old: Max days old for last integration,
-                                if exists. Default: 30 days old.
-        :param timeout: Time in seconds before timeout connection. Default: 60 seconds.
+                                if exists.
+        :param max_periods_old: Max Months old for last integration,
+                                if exists.
+        :param timeout: Time in seconds before timeout connection.
         :return: DataSource Data Object
         """
+        self.tax_id = tax_id
+        cnpj = CNPJ()
+        if not cnpj.validate(self.tax_id):
+            raise ValueError("Invalid TaxId")
+        self.tax_id = only_numbers(self.tax_id)
+
         if not timeout:
             timeout = self.default_timeout
 
         if not max_days_old:
             max_days_old = self.default_max_days
 
+        if not max_periods_old:
+            max_periods_old = self.default_max_periods
+
         data = self._last_prospect(timeout=timeout)
-        data.find_current_status(max_days_old=max_days_old)
+        data.find_current_status(
+            max_days_old=max_days_old, max_periods_old=max_periods_old
+        )
+        logger.info(f"Data received. Status is: {data.integration_status}")
 
         # Check Status
         if data.integration_status in ["SUCCESS", "TIMEOUT", "CONN_ERROR"]:
             return data
 
         if data.integration_status == "WAITING":
+            logger.info(f"Waiting for new data... Timeout is: {timeout} seconds.")
             wait_data = self._waiter(timeout=timeout)
             return wait_data
 
